#include "mbed.h"
#include "rtos.h"
#include "buffered_serial.h"
#include "ff.h"
#include "BMP085.h"

I2C i2c(p9, p10); // sda, scl
BMP085 alt_sensor(i2c);

const float BAT_GPS_MUL = 15.51;
const float BAT_MBED_MUL = 10.26;

Serial pc(USBTX, USBRX);
BufferedSerial gps(NC, p14);
AnalogIn gps_battery(p20);
AnalogIn mbed_battery(p19);

typedef struct {
    char    line[80];
} gps_line;
MemoryPool<gps_line, 16> mpool_gps_line;
Queue<gps_line, 16> queue_gps_line;

typedef struct {
    char line[80];
} sensor_line;
MemoryPool<sensor_line, 16> mpool_sensor_line;
Queue<sensor_line, 16> queue_sensor_line;

void gps_thread(void const *args)
{
    char buffer[80];

    DigitalOut gps_led(LED4);

    gps.baud(4800);

    while(true) {
        gps_led = 1;
        gps.read_line(buffer);
        //pc.puts(buffer);
        // send to log...
        gps_led = 0;
        gps_line *message = mpool_gps_line.alloc();
        strcpy(message->line, buffer);
        queue_gps_line.put(message);
    }
}

void log_thread(const void *args)
{
    FATFS fs;
    FIL fp_gps, fp_sensor;

    DigitalOut log_led(LED3);

    f_mount(0, &fs);
    f_open(&fp_gps, "0:gps.txt", FA_CREATE_ALWAYS | FA_WRITE);
    f_open(&fp_sensor, "0:sensor.txt", FA_CREATE_ALWAYS | FA_WRITE);

    while(1) {
        log_led = 1;
        osEvent evt1 = queue_gps_line.get(1);
        if (evt1.status == osEventMessage) {
            gps_line *message = (gps_line*)evt1.value.p;

            f_puts(message->line, &fp_gps);
            f_sync(&fp_gps);
            
            mpool_gps_line.free(message);
        }
        
        osEvent evt2 = queue_sensor_line.get(1);
        if(evt2.status == osEventMessage) {
            sensor_line *message = (sensor_line*)evt2.value.p;
            
            f_puts(message->line, &fp_sensor);
            f_sync(&fp_sensor);
            
            mpool_sensor_line.free(message);
        }
        log_led = 0;
    }
}

void sensor_thread(const void* args)
{
    DigitalOut sensor_led(LED2);
    Timer t;
    t.start();
    
    sensor_line *message = mpool_sensor_line.alloc();
    strcpy(message->line, "Time(s),GPS Battery(V),mbed Battery(V),BMP085 Temperature(C),Pressure,Altitude(ft)\r\n");
    queue_sensor_line.put(message);
    
    while(true)
    {
        //get sensor line memory
        sensor_led = 1;
        sensor_line *message = mpool_sensor_line.alloc();
        
        //timestamp
        float time = t.read();
        
        //gps battery
        float gps_battery_voltage = gps_battery.read()*BAT_GPS_MUL;
        
        //mbed battery
        float mbed_battery_voltage = mbed_battery.read()*BAT_MBED_MUL;
        
        //BMP085
        float bmp_temperature = (float)alt_sensor.get_temperature() / 10.0;
        int bmp_pressure = alt_sensor.get_pressure();
        float bmp_altitude = alt_sensor.get_altitude_ft();
                        
        sprintf(message->line, "%f,%f,%f,%f,%d,%f\r\n", time,gps_battery_voltage,mbed_battery_voltage,bmp_temperature,bmp_pressure,bmp_altitude);
        queue_sensor_line.put(message);
        sensor_led = 0;        
    }
}

int main()
{
    pc.baud(9600);
    Thread thread(gps_thread, NULL, osPriorityHigh);
    Thread thread2(log_thread, NULL, osPriorityNormal);
    Thread thread3(sensor_thread, NULL, osPriorityNormal);

    while(true) {
    }
}